@@ -35,7 +35,7 @@ enum connman_policy {
 };
 
 static enum connman_policy global_policy = CONNMAN_POLICY_SINGLE;
-static connman_bool_t global_flightmode = FALSE;
+static connman_bool_t global_offlinemode = FALSE;
 
 static const char *policy2string(enum connman_policy policy)
 {
@@ -81,9 +81,31 @@ static void append_profiles(DBusMessageIter *dict)
 
 	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
 				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
 	__connman_profile_list(&iter);
+	dbus_message_iter_close_container(&value, &iter);
 
+	dbus_message_iter_close_container(&entry, &value);
+
+	dbus_message_iter_close_container(dict, &entry);
+}
+
+static void append_services(DBusMessageIter *dict)
+{
+	DBusMessageIter entry, value, iter;
+	const char *key = "Services";
+
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
+								NULL, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
+								&value);
+
+	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
+				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
+	__connman_profile_list_services(&iter);
 	dbus_message_iter_close_container(&value, &iter);
 
 	dbus_message_iter_close_container(&entry, &value);
@@ -107,9 +129,7 @@ static void append_devices(DBusMessageIter *dict)
 
 	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
 				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
 	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, &iter);
-
 	dbus_message_iter_close_container(&value, &iter);
 
 	dbus_message_iter_close_container(&entry, &value);
@@ -133,9 +153,7 @@ static void append_connections(DBusMessageIter *dict)
 
 	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
 				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
 	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION, &iter);
-
 	dbus_message_iter_close_container(&value, &iter);
 
 	dbus_message_iter_close_container(&entry, &value);
@@ -167,7 +185,13 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
+	str = __connman_profile_active();
+	if (str != NULL)
+		connman_dbus_dict_append_variant(&dict, "ActiveProfile",
+						DBUS_TYPE_OBJECT_PATH, &str);
+
 	append_profiles(&dict);
+	append_services(&dict);
 
 	append_devices(&dict);
 	append_connections(&dict);
@@ -185,8 +209,8 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_variant(&dict, "Policy",
 						DBUS_TYPE_STRING, &str);
 
-	connman_dbus_dict_append_variant(&dict, "FlightMode",
-					DBUS_TYPE_BOOLEAN, &global_flightmode);
+	connman_dbus_dict_append_variant(&dict, "OfflineMode",
+				DBUS_TYPE_BOOLEAN, &global_offlinemode);
 
 	dbus_message_iter_close_container(&array, &dict);
 
@@ -222,22 +246,54 @@ static DBusMessage *set_property(DBusConnection *conn,
 			return __connman_error_invalid_arguments(msg);
 
 		global_policy = policy;
-	} else if (g_str_equal(name, "FlightMode") == TRUE) {
-		connman_bool_t flightmode;
+	} else if (g_str_equal(name, "OfflineMode") == TRUE) {
+		connman_bool_t offlinemode;
 
-		dbus_message_iter_get_basic(&value, &flightmode);
+		dbus_message_iter_get_basic(&value, &offlinemode);
 
-		if (global_flightmode == flightmode)
+		if (global_offlinemode == offlinemode)
 			return __connman_error_invalid_arguments(msg);
 
-		global_flightmode = flightmode;
+		global_offlinemode = offlinemode;
 
-		__connman_device_set_flightmode(flightmode);
+		__connman_device_set_offlinemode(offlinemode);
+	} else if (g_str_equal(name, "ActiveProfile") == TRUE) {
+		const char *str;
+
+		dbus_message_iter_get_basic(&value, &str);
+
+		return __connman_error_not_supported(msg);
 	}
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static DBusMessage *add_profile(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *name;
+
+	DBG("conn %p", conn);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
+							DBUS_TYPE_INVALID);
+
+	return __connman_error_not_supported(msg);
+}
+
+static DBusMessage *remove_profile(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *path;
+
+	DBG("conn %p", conn);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
+							DBUS_TYPE_INVALID);
+
+	return __connman_error_not_supported(msg);
+}
+
 static DBusMessage *register_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -289,6 +345,8 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 static GDBusMethodTable manager_methods[] = {
 	{ "GetProperties",   "",   "a{sv}", get_properties   },
 	{ "SetProperty",     "sv", "",      set_property     },
+	{ "AddProfile",      "s",  "o",     add_profile      },
+	{ "RemoveProfile",   "o",  "",      remove_profile   },
 	{ "RegisterAgent",   "o",  "",      register_agent   },
 	{ "UnregisterAgent", "o",  "",      unregister_agent },
 	{ },
